package es.ucm.fdi.grasia.faerie.core;

import java.io.IOException;
import java.net.URL;

import es.ucm.fdi.grasia.faerie.core.model.BundleEntity;

/**
 * It provides clients with methods to access the context, as well as other methods of the framework related with the
 * lifecycle of the components.
 * 
 * @author Chema
 * @see Context
 */
public interface ContextBroker {

	/**
	 * Returns a reference to the context of the environment.
	 * 
	 * @return a reference to the context of the environment.
	 */
	public Context getContext();

	/**
	 * Returns the entity representing the calling service component.
	 * 
	 * @return the entity representing the calling service component.
	 */
	public BundleEntity getBundleEntity();

	public Entity installBundle(URL bundleURL) throws IOException;

	public void uninstallBundle(Entity bundleEntity);

	public String instantiateComponent(String componentName);

	public void disposeComponent(String componentObject);
}
