package es.ucm.fdi.grasia.faerie.core;

/**
 * Represents a component running in the smart environment. It provides lifecycle methods and has access to framework
 * via {@code ContextManager}
 * 
 * @author Chema
 * @see ContextManager
 */
public interface ContextComponent {

	/**
	 * Callback method used to inject the {@code ContextBroker} to the service component.
	 * 
	 * @param contextBroker the {@code ContextBroker} for the component.
	 */
	public void bindContextBroker(ContextBroker contextBroker);

	/**
	 * Callback method used when the service component is activated.
	 */
	public void activate();

	/**
	 * Callback method used when the service component is deactivated.
	 */
	public void deactivate();

	/**
	 * Callback method used to detach the {@code ContextBroker} from the component.
	 * 
	 * @param contextBroker the {@code ContextBroker} being detached.
	 */
	public void unbindContextBroker(ContextBroker contextBroker);
}
