package es.ucm.fdi.grasia.faerie.core;

import java.io.Serializable;
import java.net.URI;

/**
 * ! hecho
 *
 * Represents a single (possibly abstract or compound) element of the context, which changes can be observed.
 *
 * @author Chema
 * @see ContextObserverCallback
 */
public interface ContextElement extends Serializable {

	/**
	 * Returns the identifier of this element. It has the following syntax:
	 *
	 * {@code  context://<authority>/<namespace>/<instance>[#<attribute> | ?<relationship>]}
	 *
	 * @return the identifier of this element with the specified syntax.
	 */
	public URI getId();

	/**
	 * Adds an observer to be notified for the changes in this element.
	 *
	 * @param observer the observer to be added.
	 * @return true if the observer was not added before, false otherwise.
	 */
	public boolean addObserver(ContextObserverCallback observer);

	/**
	 * Removes an observer from being notified for the changes in this element.
	 *
	 * @param observer the observer to be removed.
	 * @return true if the observer was added before, false otherwise.
	 */
	public boolean removeObserver(ContextObserverCallback observer);

	/**
	 * Returns the number of {@code ContextObserverCallback}s added to this element.
	 *
	 * @return the number of {@code ContextObserverCallback}s added to this element.
	 */
	public int countObservers();

	public Context getContext();
}
