package es.ucm.fdi.grasia.faerie.core;

public interface ContextObserverCallback {

	public static final int HANDLE_NONE = 0;
	public static final int HANDLE_READ = 1;
	public static final int HANDLE_WRITE = 2;
	public static final int HANDLE_READ_WRITE = 3;

	/**
	 * This method is implemented by an observer in order for the {@code ContextElement}s to determine if a certain
	 * event is handled by this observer.
	 *
	 * @param event the event to test if is handled by this observer.
	 * @return true if this observer handles the event, false otherwise.
	 */
	public int handle(ContextEvent event);

	/**
	 * This method is called by the {@code Context} when a new {@code ContextElement} is added on the system.
	 *
	 * @param event the event associated of type {@link ContextEvent#ELEMENT_ADDED}.
	 */
	public void elementAdded(ContextEvent event);

	/**
	 * This method is called by the {@code Context} when a {@code ContextElement} is removed from the system.
	 *
	 * @param event the event associated of type {@link ContextEvent#ELEMENT_REMOVED}.
	 */
	public void elementRemoved(ContextEvent event);

	/**
	 * This method is called by an observed {@code ContextElement} when it state changes. The change is specified by the
	 * associated {@code ContextEvent}.
	 *
	 * @param event the associated {@code ContextEvent}.
	 */
	public void elementUpdated(ContextEvent event);

}
