package es.ucm.fdi.grasia.faerie.core;

import java.io.Serializable;
import java.util.Date;

/**
 * Represents the value of an {@code AttributeInstance} in a certain moment.
 *
 * @author Chema
 * @param <T> the type of the data.
 * @see Attribute
 */
public interface ContextValue<T> extends Serializable {

	/**
	 * Returns the associated data of the {@code ContextValue}
	 *
	 * @return the associated data.
	 */
	public T getData();

	/**
	 * Returns the time when the data was obtained.
	 *
	 * @return the time when the data was obtained.
	 */
	public Date getTimeStamp();

	/**
	 * Returns a real number in (0.0, 1.0] representing the quality of the measuring of the data.
	 *
	 * @return a real number in (0.0, 1.0] representing the quality of the measuring of the data.
	 */
	public double getQuality();

	/**
	 * Returns the source wich produced the data.
	 *
	 * @return the source wich produced the data.
	 */
	public <E extends Entity> E getSource();

	public boolean equalsWithoutDate(ContextValue contextValue);
}
