package es.ucm.fdi.grasia.faerie.core;

import java.net.URI;
import java.util.Map;

/**
 * ! hecho
 * 
 * Represents an (possibly compound or abstract) entity in the smart environment that has attributes and holds
 * relationships with other entities.
 * 
 * @author Chema
 * @see Attribute
 * @see Relationship
 */
@SuppressWarnings({ "rawtypes" })
public interface Entity extends ContextElement {

	/**
	 * Returns the authority URI of this entity, i.e., the URI of the creating container.
	 * 
	 * @return the authority URI of this entity.
	 */
	public URI getAuthority();

	/**
	 * Returns the namespace URI of this entity, i.e., the URI of the creating application.
	 * 
	 * @return the namespace URI of this entity.
	 */
	public URI getNamespace();

	/**
	 * Returns the name URI of this entity.
	 * 
	 * @return the name URI of this entity.
	 */
	public URI getName();

	/**
	 * Returns the concatenation of namespace and name URIs of this entity.
	 * 
	 * @return the concatenation of namespace and name URIs of this entity.
	 */
	public URI getLocalId();

	/**
	 * Returns the {@code Attribute(Instance)} of the specified class, associated with this entity.
	 * 
	 * @param <A> the {code Attribute(Instance)} class.
	 * @param attribute the specified {code Attribute(Instance)} class object.
	 * @return the {@code Attribute(Instance)} of the specified class, associated with this entity.
	 */
	public <A extends Attribute> A getAttributeInstance(Class<A> attribute);

	/**
	 * Returns all the {@code Attribute(Instance)}s associated with this entity, mapped by their class objects.
	 * 
	 * @return all the {@code Attribute(Instance)}s associated with this entity, mapped by their class objects.
	 */
	public Map<Class<? extends Attribute>, Attribute> getAllAttributesInstances();

	/**
	 * Associates the specified {@code Attribute(Instance)} to this entity.
	 * 
	 * @param <A> the {@Attribute(Instance)} class.
	 * @param attributeInstance the specified {@code Attribute(Instance)}.
	 * @return the previously associated {@code Attribute(Instance)} of the same class, or null if it does not exists.
	 */
	public <A extends Attribute> A putAttributeInstance(A attributeInstance);

	/**
	 * Removes the {@code Attribute(Instance)} of the specified class from this entity.
	 * 
	 * @param <A> the {@Attribute(Instance)} class.
	 * @param attribute the specified {@code Attribute(Instance)} class object.
	 * @return the removed {@code Attribute(Instance)}.
	 */
	public <A extends Attribute> A removeAttributeInstance(Class<A> attribute);

	/**
	 * Removes all associated {@code Attribute(Instance)}s from this entity.
	 */
	public void removeAllAttributesInstances();

	/**
	 * Returns the {@code Relationship(Instance)} of the specified class, associated with this entity.
	 * 
	 * @param <R> the {@code Relationship(Instance)} class.
	 * @param relationship the specified {@code Relationship(Instance)} class object.
	 * @return the {@code Relationship(Instance)} of the specified class, associated with this entity.
	 */
	public <R extends Relationship> R getRelationshipInstance(Class<R> relationship);

	/**
	 * Returns all the {@code Relationship(Instance)}s associated with this entity, mapped by their class objects.
	 * 
	 * @return all the {@code Relationship(Instance)}s associated with this entity, mapped by their class objects.
	 */
	public Map<Class<? extends Relationship>, Relationship> getAllRelationshipsInstances();

	/**
	 * Associates the specified {@code Relationship(Instance)} to this entity.
	 * 
	 * @param <R> the {@code Relationship(Instance)} class.
	 * @param relationshipInstance the specified {@code Relationship(Instance)}.
	 * @return the previously associated {@code Relationship(Instance)} of the same class, or null if it does not
	 *         exists.
	 */
	public <R extends Relationship> R putRelationshipInstance(R relationshipInstance);

	/**
	 * Removes the {@code Relationship(Instance)} of the specified class from this entity.
	 * 
	 * @param <R> the {@code Relationship(Instance)} class.
	 * @param relationship the specified {@code Relationship(Instance)} class object.
	 * @return the removed {@code Relationship(Instance)}.
	 */
	public <R extends Relationship> R removeRelationshipInstance(Class<R> relationship);

	/**
	 * Removes all associated {@code Relationship(Instance)}s from this entity.
	 */
	public void removeAllRelationshipsInstances();
}
